PROGRAM zrp
  ! **************************************************************************
  !
  !  PURPOSE
  !  Simulates the zero-range process with rates
  !
  !    g(0) = 0  and  g(k) = 1 + b/k, k >= 1
  !
  !  on a one-dimensional torus with symmetric nearest-neighbor transitions
  !
  !  ARGUMENTS
  !  number_particle - INTEGER
  !  number_site     - INTEGER
  !  occupation      - REAL(number_site)
  !  b               - REAL
  !  time_in         - INTEGER (number of cycles)
  !  time_out        - INTEGER (iteration within a cycle)
  !
  !  RETURN
  !  evolution       - ARRAY INTEGER
  !
  !  CONTRIBUTOR
  !  Michail Loulakis, 12. October 2020
  !
  !  REVISIONS
  !
  !  -
  !
  !  KEYWORDS
  !
  ! **************************************************************************
  !
  IMPLICIT NONE
  !
  ! .. Parameters ..
  INTEGER              :: number_particle=200, number_site=5
  INTEGER              :: time_out=2000, time_in=25
  REAL                 :: b=
  ! .. Internal Scalars ..
  INTEGER              :: i, j
  LOGICAL              :: updating
  REAL                 :: acceptance
  !
  ! .. Internal Arrays ..
  INTEGER              :: choose_site(2)
  INTEGER, ALLOCATABLE :: occ(:)
  REAL, ALLOCATABLE    :: weight(:)
  !
  ! .. ALLOCATE arrays ..
  ALLOCATE( occ(number_site) )
  ALLOCATE( weight(number_site) )
  !
  ! .. Set initial state
  DO i = 1, number_site
  !  .. Remove this .. occ(i) = 1
    occ(i) = 0
  END DO
  !  .. Remove this ..occ(INT(number_site/2)) = number_particle - number_site + 1
  DO i = 1, 5
    occ(i) =40
  END DO
  !
  ! .. Store data in file ..
  OPEN(UNIT=30, FILE="data/data_N200_L5_b3.dat", ACTION="write")
  WRITE(30,*) occ
  !
  ! .. Compute  weights ..
  DO i = 1, number_site
     IF (occ(i) .EQ. 0) THEN
        weight(i) = 0
     ELSE
        weight(i) = ( 1 + b/occ(i) ) / (1 + b)
     END IF
  END DO
  !
  ! .. Iteration ..
  DO i = 1, time_out
     !
     ! .. Update within a cyle
     DO j = 1, time_in
       !
       ! .. Choose lattice site according to the weights g(k)/(1+b)
       updating = .TRUE.
       DO WHILE (updating)
          choose_site(1) = INT(RAND(0)*number_site) + 1
          acceptance     = weight( choose_site(1) )
          IF ( RAND(0) .LT. acceptance) THEN 
             updating = .FALSE. 
          END IF
       END DO
       !
       ! .. Choose random walk step
       choose_site(2) = choose_site(1) + 2*INT( RAND(0) + .5 ) - 1
       IF (choose_site(2) .GT. number_site) THEN 
          choose_site(2) = 1
       END IF
       IF (choose_site(2) .LT. 1) THEN
          choose_site(2) = number_site
       END IF
       !     
       ! .. Update occupation_site and weight
       occ(choose_site(1)) = occ(choose_site(1)) - 1
       occ(choose_site(2)) = occ(choose_site(2)) + 1
       !     
       IF (occ(choose_site(1)) .EQ. 0) THEN
          weight(choose_site(1)) = 0
       ELSE
          weight(choose_site(1)) &
               = ( 1 + b/occ(choose_site(1)) ) / (1 + b)
       END IF
       !
       weight(choose_site(2)) &
            = ( 1 + b/occ(choose_site(2)) ) / (1 + b)
    END DO
    !
    WRITE(30, *) occ
    PRINT *, i
    !
  END DO
  CLOSE(30)
END PROGRAM zrp
